/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.framework;

import java.awt.AWTException;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;



import javax.imageio.ImageIO;



/**
 * Class used for automatic mouse and keyboard system events generation. This
 * class also contains methods for capturing the screenshot and for finding some
 * pattern in that screenshot.
 * 
 * @author Pavel Tisnovsky
 */
public class GuiRobot
{

    /**
     * Number of millisecond between two consecutive robot actions.
     */
    private static final int ROBOT_DELAY = 100;

    /**
     * Number of millisecond for key press event. 
     */
    private static final int KEY_PRESS_DELAY = 10;

    Robot robot = null;

    BufferedImage screenCapture = null;

    public GuiRobot() throws AWTException
    {
        this.robot = new Robot();
    }

    /**
     * @return the robot
     */
    public Robot getRobot()
    {
        return this.robot;
    }

    /**
     * @param robot the robot to set
     */
    public void setRobot(Robot robot)
    {
        this.robot = robot;
    }

    /**
     * @return the screenCapture
     */
    public BufferedImage getScreenCapture()
    {
        return this.screenCapture;
    }

    /**
     * @param screenCapture the screenCapture to set
     */
    public void setScreenCapture(BufferedImage screenCapture)
    {
        this.screenCapture = screenCapture;
    }

    public void createScreenCapture()
    {
        Rectangle screenRectangle = new Rectangle(Toolkit.getDefaultToolkit().getScreenSize());
        setScreenCapture(getRobot().createScreenCapture(screenRectangle));
    }

    public void saveScreenshot(String fileName) throws IOException
    {
        ImageIO.write(getScreenCapture(), "png", new File("screenshots/" + fileName + ".png"));
    }

    public void saveImage(String fileName) throws IOException
    {
        ImageIO.write(getScreenCapture(), "png", new File(fileName));
    }

    public void loadScreenshot(String fileName) throws IOException
    {
        loadImage("screenshots/" + fileName + ".png");
    }

    public void loadImage(String fileName) throws IOException
    {
        setScreenCapture(ImageIO.read(new File(fileName)));
    }

    public Rectangle findPattern(String markerFileName) throws IOException
    {
        BufferedImage marker = ImageIO.read(new File(markerFileName));
        return ImageProcessing.findPattern(marker, getScreenCapture());
    }

    public void highlightRectangle(Rectangle rect)
    {
        highlightRectangle(getScreenCapture(), rect);
    }

    public void clickToRectangle(Rectangle rectangle)
    {
        if (rectangle == null)
        {
            return;
        }

        int x = rectangle.x + rectangle.width / 2;
        int y = rectangle.y + rectangle.height / 2;
        click(x, y);
    }

    /**
     * @param x
     * @param y
     */
    private void click(int x, int y) {
        this.robot.mouseMove(x, y);
        this.robot.delay(ROBOT_DELAY);
        this.robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
        this.robot.delay(ROBOT_DELAY);
        this.robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);
    }

    public void pressKey(int keycode)
    {
        robot.keyPress(keycode);
        robot.delay(KEY_PRESS_DELAY);
        robot.keyRelease(keycode);
    }

    /**
     * Method press2Keys
     * presses key2 while key1 is pressed, then release both.
     * Can be used e.g. to press key1=CTRL and key2=S to get CTRL+S.
     * 
     * @param key1
     * @param key2
     */
    public void press2Keys(int key1, int key2)
    {
        robot.keyPress(key1);
        robot.delay(KEY_PRESS_DELAY);
        robot.keyPress(key2);
        robot.delay(KEY_PRESS_DELAY);
        robot.keyRelease(key2);
        robot.delay(KEY_PRESS_DELAY);
        robot.keyRelease(key1);
    }
    
    public void pressCtrlPlusKey(int keycode)
    {
        press2Keys(KeyEvent.VK_CONTROL, keycode);
    }

    /**
     * Method resize
     * gets a small rectangle where the right lower corner of the gui is,
     * and number of x and y pixels by which the window should be resized.
     * It uses mouse and resizes the gui window, while adding sufficient
     * time delays between the mouse actions.
     * 
     * @param from
     * @param xd
     * @param yd
     */
    public void resize(Rectangle from, int xd, int yd)
    {
        int x = from.x + from.width - 4;
        int y = from.y + from.height - 4;
        robot.mouseMove(x, y);
        robot.delay(ROBOT_DELAY);
        x += 4;
        y += 4;
        robot.mouseMove(x, y);
        robot.delay(ROBOT_DELAY);
        robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
        robot.delay(ROBOT_DELAY);
        robot.mouseMove(x+xd, y+yd);
        robot.delay(ROBOT_DELAY);
        robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);
    }
    
    /**
     * Add some offset to a rectangle.
     * 
     * @param rect
     *            input rectangle (it's not changed)
     * @param offset
     *            offset, if greater than zero, the new rectangle is bigger than
     *            original one
     * @return new rectangle
     */
    private static Rectangle getRectangleWithOffset(Rectangle rect, int offset)
    {
        int x = rect.x - offset;
        int y = rect.y - offset;
        int width = rect.width + (offset << 1);
        int height = rect.height + (offset << 1);
        return new Rectangle(x, y, width, height);
    }

    /**
     * Highlight rectangle around the marker in the tested image.
     * 
     * @param testImage
     *            tested image
     * @param rect
     *            rectangle around the marker found in the tested image
     */
    private static void highlightRectangle(BufferedImage testImage, Rectangle rect)
    {
        Graphics2D graphics = (Graphics2D) testImage.getGraphics();
        Rectangle rect1 = getRectangleWithOffset(rect, 2);
        Rectangle rect2 = getRectangleWithOffset(rect, 4);
        graphics.setColor(Color.RED);
        graphics.draw(rect1);
        graphics.setColor(Color.BLACK);
        graphics.draw(rect2);
    }

}
